/* by Luis Figueiredo (stdio@netc.pt)
 *
 *
 * date: Sat Mar 30 14:25:25 GMT 2002 
 *
 *  memory functions
 */
#include "memory.h"

IFDEBUG(int _t=0;)
void * mymalloc(size_t s) {
	void *ret;
	ret=malloc(s);
	if(ret==NULL) {
		IFDEBUG(fprintf(stderr,"memory.c: malloc: %s (size %d)\n",strerror(errno),s));
		return NULL;
	};
	IFDEBUG(_t++;);
	IFDEBUG(fprintf(stderr,"memory.c: Allocated (%d) %d bytes to %p\n",_t,s,ret));
	return ret;
}
void * mycalloc(size_t nmemb,size_t s) {
	void *ret;
	ret=calloc(nmemb,s);
	if(ret==NULL) {
		IFDEBUG(fprintf(stderr,"memory.c: calloc %s\n",strerror(errno)));
		return NULL;
	};
	IFDEBUG(_t++;);
	IFDEBUG(fprintf(stderr,"memory.c: Allocated (%d) %d bytes to %p\n",_t,s*nmemb,ret));
	return ret;
}
void * myrealloc(void *buf,size_t s) {
	void *ret;
	ret=realloc(buf,s);
#ifdef DEBUG
	if(buf==NULL) {
		_t++;
		IFDEBUG(fprintf(stderr,"memory.c: Allocated (%d) %d bytes to %p\n",_t,s,ret));
	};
#endif
	if(ret==NULL) {
		IFDEBUG(fprintf(stderr,"memory.c: realloc: %s\n",strerror(errno)));
		return NULL;
	};
	IFDEBUG(fprintf(stderr,"memory.c: Realloc buffer %p to %d\n",buf,s));
	return ret;
}

void myfree(void *ptr) {
	if(ptr!=NULL) {
		free(ptr);
		IFDEBUG(fprintf(stderr,"memory.c: Buffer (%d) %p freed\n",_t,ptr));
		IFDEBUG(_t--;);
	};
}



void *add_buffer(struct memrequest *list,unsigned int size) {
	struct memrequest *tmem;
	if(size==0) {
		return NULL;
	};
	if(list!=NULL) {
		tmem=list;
	}else {
		return NULL;
	};
	while(tmem->next!=NULL)tmem=tmem->next;
	IFDEBUG(fprintf(stderr,"memory.c: (allocating mem buffer node): "));
	tmem->next=mymalloc(sizeof(struct memrequest));
	IFDEBUG(fprintf(stderr,"memory.c: (allocating mem buffer): "));
	tmem->next->ptr=mymalloc(size);
	tmem->next->next=NULL;
	return tmem->next->ptr;
}
struct memrequest *init_buffer_list() {
	struct memrequest *newlist;
	IFDEBUG(fprintf(stderr,"memory.c: (allocating list): "));
	newlist=mymalloc(sizeof(struct memrequest));
	newlist->next=NULL;
	newlist->ptr=NULL;
	return newlist;
}

void delete_buffer(struct memrequest *mem) {
	IFDEBUG(fprintf(stderr,"memory.c: (freeing mem buffer): "));
	myfree(mem->ptr);
	IFDEBUG(fprintf(stderr,"memory.c: (freeing mem buffer node): "));
	myfree(mem);
}
void delete_next_buffer(struct memrequest *mem) {
	struct memrequest *tmem;
	tmem=mem->next;
	mem->next=mem->next->next;
	delete_buffer(tmem);
}
void delete_buffer_list(struct memrequest *list) {
	struct memrequest *tmem=list;
	if(tmem==NULL) return;

	while(tmem->next!=NULL) {
		delete_next_buffer(tmem);
	};
	delete_buffer(tmem);
}

